@@ -316,17 +316,20 @@ class EnvReloader:
             w.calls = FilesWatcher.Callbacks(on_event=self.calls.on_env_edit)
             self.env_watchers.append(w)
 
-        for p in self.li.shell_env.env.get_user_envs():
-            watcher = FilesWatcher(
-                FilesWatcher.Sets(
-                    root=p.Meta.root,
-                    include=self.se.watch_files + ["env_*.py"],
-                    exclude=self.se.ignore_files + [r"**/.*", r"**/*~", r"**/__pycache__"],
-                    name=p.__name__,
-                ),
-                calls=FilesWatcher.Callbacks(on_event=self.calls.on_env_edit),
-            )
-            self.env_watchers.append(watcher)
+        try:
+            for p in self.li.shell_env.env.get_user_envs():
+                watcher = FilesWatcher(
+                    FilesWatcher.Sets(
+                        root=p.Meta.root,
+                        include=self.se.watch_files + ["env_*.py"],
+                        exclude=self.se.ignore_files + [r"**/.*", r"**/*~", r"**/__pycache__"],
+                        name=p.__name__,
+                    ),
+                    calls=FilesWatcher.Callbacks(on_event=self.calls.on_env_edit),
+                )
+                self.env_watchers.append(watcher)
+        except OSError:
+            logger.warning("Could not create files watcher. Reloading will not work")
 
     def start(self) -> None:
         self.li.status.reloader_ready = True
@@ -389,7 +392,6 @@ class Env(BaseEnv):
     _environ_before: Optional[Dict[str, str]]
 
     env_id_to_secrets: ClassVar[Dict[str, Secrets]] = {}
-
     _shell: "Shell"
 
     def __init__(self):
@@ -565,8 +567,6 @@ class ShellEnv:
     magic_functions: Dict[str, Any]
     env: Env
 
-    first_run = True
-
     def __init__(self, calls: _Callbacks, se: _Sets, li: _Links) -> None:
         self._calls = calls
         self._se = se
@@ -680,15 +680,12 @@ class ShellEnv:
 
             sw = Stopwatch()
             sw.start()
-            oncreate_funs = self.magic_functions["oncreate"].values()
+
             onload_funs = self.magic_functions["onload"].values()
 
             self._start_reloaders()
 
             try:
-                if self.first_run:
-                    for h in oncreate_funs:
-                        h()
                 for h in onload_funs:
                     h()
                 self._run_boot_codes()
@@ -717,6 +714,15 @@ class ShellEnv:
         else:
             thread(self)
 
+    def on_shell_create(self) -> None:
+        """
+        Called only after creation.
+        :return:
+        """
+        functions = self.magic_functions["oncreate"].values()
+        for h in functions:
+            h()
+
     def _collect_magic_functions(self) -> None:
         """
         Go through fields and transform decorated functions to commands.
